﻿-- UIEffects.lua
-- Implements expand-in/expand-out effects
-- Based on Blizzard's UIFrameFadeIn/UIFrameFadeOut code

--[[
DOCUMENTATION
Hint: These functions accept the same args as Blizzard's UIFrameFadeIn and
UIFrameFadeOut functions.

Torta.gui:ScaleIn(frame, timeToResize, startScale, endScale)
frame        - The frame to expand
timeToResize - Time that it will take to go from startScale to endScale
startScale   - Width to start expanding from. Must be a number smaller than endScale.
endScale     - Final width of the frame. Must be a number larger than startScale.

Torta.gui:ScaleOut(frame, timeToResize, startScale, endScale)
frame        - The frame to expand
timeToResize - Time that it will take to go from startScale to endScale
startScale   - Width to start expanding from. Must be a number larger than endScale.
endScale     - Final width of the frame. Must be a number smaller than startScale.

Torta.gui:IsFrameScaling(frame)
frame        - Frame to check if it is scaling
  RETURNS:   1 if frame is being scaled, otherwise nil.

Torta.gui:RemoveScalingFrame(frame)
frame        - Frame to stop being rescaled
--]]

local SCALEFRAMES = {};

-- The function that'll do the actual registration
function Torta.gui:UIFrameScale(frame, scaleInfo)
  if (not frame) then
    return;
  end

  local index = 1;
  --[[
  while SCALEFRAMES[index] do
    -- If frame is already set to rescale then return
    if ( SCALEFRAMES[index] == frame ) then
      return;
    end
    index = index + 1;
  end
  ]]
  frame:SetScale(scaleInfo.startScale);

  frame.scaleInfo = scaleInfo;
  frame:Show();
  tinsert(SCALEFRAMES, frame);
end

-- Expand function
function Torta.gui:ScaleIn(frame, timeToResize, startScale, endScale)
  assert(frame, "Torta.gui:ScaleInWidth - Missing arg#1 \"frame\"");
  assert(endScale >= startScale, "endScale is smaller than startScale - use Torta.gui:ScaleOut for this")
  
  local scaleInfo = {};
  scaleInfo.mode = "EXPAND";
  scaleInfo.timeToResize = timeToResize or error("Torta.gui:ScaleIn - Missing arg#2 \"timeToResize\"");
  scaleInfo.startScale = startScale or error("Torta.gui:ScaleIn - Missing arg#3 \"startScale\"");
  scaleInfo.endScale = endScale or error("Torta.gui:ScaleIn - Missing arg#4 \"endScale\"");
  scaleInfo.finishedFunc = frame.scalingFinishedFunc;
  Torta.gui:UIFrameScale(frame, scaleInfo);
end

-- Shrink!
function Torta.gui:ScaleOut(frame, timeToResize, startScale, endScale)
  assert(frame, "Torta.gui:ScaleOut - Missing arg#1 \"frame\"");
  assert(endScale <= startScale, "endScale is smaller than startScale - use Torta.gui:ScaleIn for this")
  
  local scaleInfo = {};
  scaleInfo.mode = "SHRINK";
  scaleInfo.timeToResize = timeToResize or error("Torta.gui:ScaleOut - Missing arg#2 \"timeToResize\"");
  scaleInfo.startScale = startScale or error("Torta.gui:ScaleOut - Missing arg#3 \"startScale\"");
  scaleInfo.endScale = endScale or error("Torta.gui:ScaleOut - Missing arg#4 \"endScale\"");
  scaleInfo.finishedFunc = frame.scalingFinishedFunc;
  Torta.gui:UIFrameScale(frame, scaleInfo);
end

function Torta.gui:RemoveScalingFrame(frame)
  tDeleteItem(SCALEFRAMES, frame);
end

-- Function that actually performs the width change
--[[
This is part is pretty much the same with the Blizzard alpha functions implementation.

Resizing frame attribute listing
============================================================
sizeInfo.timeToResize  [Num]    Time it takes to scale the frame in or out
sizeInfo.mode  ["EXPAND", "SHRINK"]  Resize mode
sizeInfo.finishedFunc [func()]  Function that is called when resize is finished
sizeInfo.finishedArg1 [ANYTHING]  Argument to the finishedFunc
sizeInfo.finishedArg2 [ANYTHING]  Argument to the finishedFunc
sizeInfo.finishedArg3 [ANYTHING]  Argument to the finishedFunc
sizeInfo.finishedArg4 [ANYTHING]  Argument to the finishedFunc
sizeInfo.fadeHoldTime [Num]  Time to hold the resized state
 ]]
function Torta.gui:UpdateScalingFrames(elapsed)
  local index = 1;
  local frame, scaleInfo;
  while SCALEFRAMES[index] do
    frame = SCALEFRAMES[index];
    scaleInfo = SCALEFRAMES[index].scaleInfo;
    
    if not scaleInfo.resizeTimer then
      scaleInfo.resizeTimer = 0;
    end
    
    scaleInfo.resizeTimer = scaleInfo.resizeTimer + elapsed;

    -- If the resizeTimer is less then the desired resize time then set the size otherwise hold the resize state, call the finished function, or just finish the resize 
    if ( scaleInfo.resizeTimer < scaleInfo.timeToResize ) then
      if ( scaleInfo.mode == "EXPAND" ) then
        frame:SetScale((scaleInfo.resizeTimer / scaleInfo.timeToResize) * (scaleInfo.endScale - scaleInfo.startScale) + scaleInfo.startScale);
      elseif ( scaleInfo.mode == "SHRINK" ) then
        frame:SetScale(((scaleInfo.timeToResize - scaleInfo.resizeTimer) / scaleInfo.timeToResize) * (scaleInfo.startScale - scaleInfo.endScale)  + scaleInfo.endScale);
      end
    else
      frame:SetScale(scaleInfo.endScale);
      -- If there is a resizeHoldTime then wait until its passed to continue on
      if ( scaleInfo.resizeHoldTime and scaleInfo.resizeHoldTime > 0  ) then
        scaleInfo.resizeHoldTime = scaleInfo.resizeHoldTime - elapsed;
      else
        -- Complete the resize and call the finished function if there is one
        self:RemoveScalingFrame(frame);
        if ( scaleInfo.finishedFunc ) then
          scaleInfo.finishedFunc(frame, scaleInfo.finishedArg1, scaleInfo.finishedArg2, scaleInfo.finishedArg3, scaleInfo.finishedArg4);
          scaleInfo.finishedFunc = nil;
        end
      end
    end
    
    index = index + 1;
  end
end

local frame = CreateFrame("Frame");
frame:SetScript("OnUpdate",
function(self, elapsed)
  Torta.gui:UpdateScalingFrames(elapsed);
end);

function Torta.gui:IsFrameScaling(frame)
  for index, value in pairs(SCALEFRAMES) do
    if ( value == frame ) then
      return 1;
    end
  end
  return nil;
end